'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _capitalize2 = require('lodash/capitalize');

var _capitalize3 = _interopRequireDefault(_capitalize2);

var _merge2 = require('lodash/merge');

var _merge3 = _interopRequireDefault(_merge2);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _createReactClass = require('create-react-class');

var _createReactClass2 = _interopRequireDefault(_createReactClass);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _loreUtils = require('lore-utils');

var _loreReactForms = require('lore-react-forms');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = (0, _createReactClass2.default)({
  displayName: 'Optimistic',

  propTypes: {
    model: _propTypes2.default.object.isRequired,
    data: _propTypes2.default.object,
    validators: _propTypes2.default.oneOfType([_propTypes2.default.func, _propTypes2.default.object]),
    onChange: _propTypes2.default.func,
    onSubmit: _propTypes2.default.func,
    callbacks: _propTypes2.default.object,
    schema: _propTypes2.default.object.isRequired,
    fieldMap: _propTypes2.default.object.isRequired,
    actionMap: _propTypes2.default.object.isRequired,
    fields: _propTypes2.default.object.isRequired,
    actions: _propTypes2.default.array.isRequired
  },

  getInitialState: function getInitialState() {
    var _props = this.props,
        data = _props.data,
        model = _props.model;


    return {
      data: (0, _merge3.default)({}, data, model.data)
    };
  },

  request: function request(data) {
    var modelName = this.props.modelName;
    var _props2 = this.props,
        model = _props2.model,
        request = _props2.request;

    return request ? request(data) : lore.actions[modelName].update(model, data).payload;
  },

  onSubmit: function onSubmit() {
    var data = this.state.data;

    this.request(data);
  },

  onChange: function onChange(name, value) {
    var nextData = (0, _merge3.default)({}, this.state.data);
    nextData[name] = value;
    this.setState({
      data: nextData
    });
  },

  getValidators: function getValidators(data) {
    var validators = this.props.validators;

    return (0, _loreUtils.result)(validators || {}, data);
  },

  render: function render() {
    var modelName = this.props.modelName;
    var _props3 = this.props,
        _props3$title = _props3.title,
        title = _props3$title === undefined ? 'Update ' + (0, _capitalize3.default)(modelName) : _props3$title,
        _props3$description = _props3.description,
        description = _props3$description === undefined ? '' : _props3$description,
        schema = _props3.schema,
        fieldMap = _props3.fieldMap,
        actionMap = _props3.actionMap,
        fields = _props3.fields,
        actions = _props3.actions;
    var data = this.state.data;

    var validators = this.getValidators(data);

    return _react2.default.createElement(
      'div',
      null,
      _react2.default.createElement(
        'div',
        { className: 'modal-header' },
        title ? _react2.default.createElement(
          'h4',
          { className: 'modal-title' },
          title
        ) : null,
        description ? _react2.default.createElement(
          'p',
          { className: 'help-block' },
          description
        ) : null
      ),
      _react2.default.createElement(_loreReactForms.SchemaForm, {
        data: data,
        validators: validators,
        onChange: this.onChange,
        callbacks: {
          onSubmit: this.onSubmit
        },
        schema: schema,
        fieldMap: fieldMap,
        actionMap: actionMap,
        fields: fields,
        actions: actions
      })
    );
  }

});